@@ -1,14 +1,10 @@
 import os
-import wget
-import time
 import json
 import yaml
-import zipfile
-import platform
 from getpass import getpass
 from subprocess import Popen, PIPE, STDOUT
 from json import JSONDecodeError
-from onepassword.utils import read_bash_return, scrape, domain_from_email, Encryption, BashProfile
+from onepassword.utils import read_bash_return, domain_from_email, Encryption, BashProfile, generate_uuid
 from onepassword.exceptions import OnePasswordForgottenPassword
 
 
@@ -16,9 +12,6 @@ class OnePassword:
     """
     Class for integrating with a One Password password manager
 
-    :param install_only: Whether to just install the cli or not (optional, default=False)
-    :type install_only: bool
-
     :param domain: domain of 1password account (optional, default=None)
     :type domain: str
 
@@ -31,123 +24,66 @@ class OnePassword:
     :param password: password of 1password account (optional, default=None)
     :type password: str
 
-    :param override_platform: name of the platform if already known (optional, default=None)
-    :type override_platform: str
-
     :returns OnePassword: :obj:`instance`: OnePassword instance to allow code or user to manage credentials.
 
     """
-    def __init__(self, install_only=False, domain=None, email=None, secret=None, password=None,
-                 override_platform=None):  # pragma: no cover
+    def __init__(self, account=None, domain=None, email=None, secret=None, password=None):  # pragma: no cover
         self.signin_domain = domain
         self.email_address = email
         self.secret_key = secret
-        self.override_platform = override_platform
-        if isinstance(override_platform, str):
-            self.system = override_platform
+        device_uuid = generate_uuid()
+        os.environ["OP_DEVICE"] = device_uuid
+        bp = BashProfile()
+        bp.update_profile("OP_DEVICE", device_uuid)
+        # Check first time: if true, full signin, else use shortened signin
+        if self.check_not_first_time(bp):
+            self.encrypted_master_password, self.session_key = self.signin_wrapper(account=account,
+                                                                                   master_password=password)
         else:
-            self.system = str(platform.system())
-        self.user_home = read_bash_return("echo $HOME")
-        # if self.system == "Linux":
-        #     self.local_bin = os.path.join(self.user_home, ".local/bin/")
-        # elif self.system == "Darwin":
-        self.local_bin = "/usr/local/bin/"
-        # Get most recent version
-        self.sub_soup, self.link, self.version = self.get_link_version()
-        self.major_version = self.version.split(".")[1]
-        # Check if installed: if true, get update, if false install
-        if self.check_cli():
-            self.encrypted_master_password, self.session_key = self.signin_wrapper(master_password=password)
-        else:
-            self.install()
-            if install_only:
-                pass
+            self.first_use()
+
+    @staticmethod
+    def check_not_first_time(bp):
+        line_checks = []
+        for line in bp.profile_lines:
+            if "OP_SESSION" in line:
+                line_checks.append(True)
             else:
-                self.first_use(master_password=password)
+                line_checks.append(False)
+        return any(line_checks)
 
-    def get_link_version(self):
-        """
-        Helper function to get the most up to date link for downloading op cli
-
-        :returns nt, download_link, version: :obj:`soup`, :obj:`str`, :obj:`str`: HTML soup, URL for download and the
-            op cli version.
-
-        """
-        download_link = ""
-        nt = scrape('https://app-updates.agilebits.com/product_history/CLI')
-        n = nt.find("h3")
-        version = n.decode_contents().split("<span")[0].split()[0]
-        dl = nt.find("p", attrs={"class": "system {}".format(self.system.lower())})
-        for d in dl.find_all("a"):
-            if d.text == "amd64":
-                download_link = d['href']
-        return nt, download_link, version
-
-    def check_cli(self):  # pragma: no cover
-        """
-        Helper function to check if op cli is already installed
-
-        :returns: :obj:`bool`: True or False
-
-         """
-        if self.override_platform == self.system:
-            return False
-        else:
-            op = read_bash_return("op --version")
-            if op == "":
-                return False
-            elif op.split(".")[1] == self.major_version:
-                return True
-            else:
-                return False
-
-    def install(self):  # pragma: no cover
-        """
-        Helper function to download, unzip, install and chmod op cli files
-
-        """
-        op_file = self.link.split("/")[-1]
-        if self.override_platform == self.system:
-            download_path = os.path.join(self.user_home, op_file)
-            final_path = self.user_home
-        else:
-            download_path = os.path.join(self.local_bin, op_file)
-            final_path = self.local_bin
-        # print('Downloading or updating the 1Password CLI: {}'.format(op_file))
-        wget.download(self.link, download_path)
-        if self.link[-4:] != ".pkg":
-            zip_ref = zipfile.ZipFile(download_path, 'r')
-            zip_ref.extractall(final_path)
-            zip_ref.close()
-            os.chmod(os.path.join(final_path, 'op'), 0o755)
-            # print("Installed: {}".format(os.path.join(self.local_bin, op_file)))
-        else:
-            Popen(["open", os.path.join(self.local_bin, op_file)], stdin=PIPE, stdout=PIPE)  # pragma: no cover
-            while self.check_cli():
-                time.sleep(1)
-
-    def first_use(self, master_password):  # pragma: no cover
+    def first_use(self):  # pragma: no cover
         """
         Helper function to perform first time signin either with user interaction or not, depending on _init_
-        :param master_password: password for 1password account (optional, default=None)
-        :type master_password: str
 
         """
-        # TODO: Could be wrapped into signin_wrapper()
-        if master_password is None:
-            email_address = input("Your email address for domain: ")
-            signin_domain = domain_from_email(email_address)
-            secret_key = getpass("1Password secret key: ")
-            self.signin_wrapper(signin_domain, email_address, secret_key)
+        email_address = input("Please input your email address used for 1Password account: ")
+        account = domain_from_email(email_address)
+        signin_domain = account + ".1password.com"
+        confirm_signin_domain = input("Is your 1Password domain: {} (y/n)? ".format(signin_domain))
+        if confirm_signin_domain == "y":
+            pass
         else:
-            self.signin_wrapper(self.signin_domain, self.email_address, self.secret_key)
+            signin_domain = input("Please input your 1Password domain in the format <something>.1password.com: ")
 
-    def signin_wrapper(self, domain=None, email=None, secret_key=None, master_password=None):  # pragma: no cover
+        confirm_account = input("Is your 1Password account name: {} (y/n)? ".format(account))
+        if confirm_account == "y":
+            pass
+        else:
+            account = input("Please input your 1Password account name e.g. wandera from wandera.1password.com: ")
+        secret_key = getpass("Please input your 1Password secret key: ")
+        self.signin_wrapper(account, signin_domain, email_address, secret_key)
+
+    def signin_wrapper(self, account=None, domain=None, email=None, secret_key=None, master_password=None):
+        # pragma: no cover
         """
         Helper function for user to sign in but allows for three incorrect passwords. If successful signs in and updates
         bash profile, if not raises exception and points user to 1Password support.
 
-        :param domain: domain of 1password account (optional, default=None)
+        :param account: shorthand name for your 1password account e.g. wandera from wandera.1password.com (optional,
+        default=None)
+
+        :param domain: full domain name of 1password account e.g. wandera.1password.com (optional, default=None)
         :type domain: str
 
         :param email: email address of 1password account (optional, default=None)
@@ -164,17 +100,21 @@ class OnePassword:
         """
         global session_key
         global encrypted_str
-        password, session_key, domain, bp = self.signin(domain, email, secret_key, master_password)
+        password, session_key, domain, account, bp = self.signin(account, domain, email, secret_key, master_password)
         tries = 1
         while tries < 3:
             if "(ERROR)  401" in session_key.decode():
                 print("That's not the right password, try again.")
-                password, session_key, domain, bp = self.signin(domain, email, secret_key, master_password)
+                password, session_key, domain, account, bp = self.signin(account, domain, email, secret_key,
+                                                                         master_password)
                 tries += 1
                 pass
             else:
-                os.environ["OP_SESSION_{}".format(domain)] = session_key.decode().replace("\n", "")
-                bp.update_profile("OP_SESSION_{}".format(domain), session_key.decode().replace("\n", ""))
+                # device_uuid = generate_uuid()
+                os.environ["OP_SESSION_{}".format(account)] = session_key.decode().replace("\n", "")
+                # os.environ["OP_DEVICE"] = device_uuid
+                bp.update_profile("OP_SESSION_{}".format(account), session_key.decode().replace("\n", ""))
+                # bp.update_profile("OP_DEVICE", device_uuid)
                 encrypt = Encryption(session_key)
                 encrypted_str = encrypt.encode(password)
                 return encrypted_str, session_key
@@ -182,11 +122,14 @@ class OnePassword:
                                            "https://support.1password.com/forgot-master-password/")
 
     @staticmethod
-    def signin(domain=None, email=None, secret_key=None, master_password=None):  # pragma: no cover
+    def signin(account=None, domain=None, email=None, secret_key=None, master_password=None):  # pragma: no cover
         """
         Helper function to prepare sign in for the user
 
-        :param domain: domain of 1password account (optional, default=None)
+        :param account: shorthand name for your 1password account e.g. wandera from wandera.1password.com (optional,
+        default=None)
+
+        :param domain: full domain name of 1password account e.g. wandera.1password.com (optional, default=None)
         :type domain: str
 
         :param email: email address of 1password account (optional, default=None)
@@ -209,30 +152,26 @@ class OnePassword:
                 encrypt = Encryption(globals()['session_key'])
                 master_password = str.encode(encrypt.decode(globals()['encrypted_str']))
             else:
-                master_password = str.encode(getpass("1Password master password: "))
+                master_password = str.encode(getpass("Please input your 1Password master password: "))
         if secret_key:
-            process = Popen(['op', "signin", domain, email, secret_key, "--output=raw"], stdout=PIPE, stdin=PIPE,
-                            stderr=STDOUT)
-            # process.stdin.write(master_password)
-            # session_key = process.communicate()[0]
-            # process.stdin.close()
-            # os.environ["OP_SESSION_{}".format(domain)] = session_key.decode().replace("\n", "")
-            # bp.update_profile("OP_SESSION_{}".format(domain), session_key.decode().replace("\n", ""))
+            process = Popen(['op', "signin", domain, email, secret_key, "--raw", "--shorthand", account], stdout=PIPE,
+                            stdin=PIPE, stderr=STDOUT)
         else:
-            if domain is None:
+            if account is None:
                 try:
                     session_dict = bp.get_key_value("OP_SESSION", fuzzy=True)[0]  # list of dicts from BashProfile
-                    domain = list(session_dict.keys())[0].split("OP_SESSION_")[1]
+                    account = list(session_dict.keys())[0].split("OP_SESSION_")[1]
                 except AttributeError:
-                    domain = input("1Password signin domain e.g. wandera.1password.com = wandera: ")
+                    account = input("Please input your 1Password account name e.g. wandera from "
+                                    "wandera.1password.com: ")
                 except ValueError:
                     raise ValueError("First signin failed or not executed.")
-            process = Popen(['op', "signin", domain, "--output=raw"], stdout=PIPE, stdin=PIPE, stderr=STDOUT)
+            process = Popen(['op', "signin", account, "--raw"], stdout=PIPE, stdin=PIPE, stderr=STDOUT)
 
         process.stdin.write(master_password)
         sess_key = process.communicate()[0]
         process.stdin.close()
-        return master_password, sess_key, domain, bp
+        return master_password, sess_key, domain, account, bp
 
     def get_uuid(self, docname, vault="Private"):  # pragma: no cover
         """
@@ -247,7 +186,7 @@ class OnePassword:
         :returns: uuid :obj:`str`: uuid of item or None if doesn't exist
 
         """
-        items = self.get_items(vault=vault)
+        items = self.list_items(vault=vault)
         for t in items:
             if t['overview']['title'] == docname:
                 return t['uuid']
@@ -363,9 +302,9 @@ class OnePassword:
         return read_bash_return('op list vaults')
 
     @staticmethod
-    def get_items(vault="Private"):
+    def list_items(vault="Private"):
         """
-        Helper function to get all items
+        Helper function to list all items in a certain vault
 
         :param vault: vault the items are in (optional, default=Private)
         :type vault: str
@@ -375,3 +314,26 @@ class OnePassword:
         """
         items = json.loads(read_bash_return("op list items --vault={}".format(vault)))
         return items
+
+    @staticmethod
+    def get_item(uuid, fields=None):
+        """
+        Helper function to get a certain field, you can find the UUID you need using list_items
+
+        :param uuid: uuid of the item you wish to get, no vault needed
+        :type uuid: str or bytes
+
+        :param fields: to return only certain detail use either a specific field or list of them
+        (optional, default=None which means all fields returned)
+        :type fields: str or bytes or list
+
+        :return: item :obj: `dict`: dict of the item with requested fields
+
+        """
+        if isinstance(fields, list):
+            item = json.loads(read_bash_return("op get item {} --fields {}".format(uuid, ",".join(fields))))
+        elif isinstance(fields, str):
+            item = {fields: read_bash_return("op get item {} --fields {}".format(uuid, fields))}
+        else:
+            item = json.loads(read_bash_return("op get item {}".format(uuid)))
+        return item
